@@ -22,8 +22,6 @@
 import("core.base.option")
 import("core.project.config")
 import("core.tool.toolchain")
-import("core.tool.linker")
-import("core.tool.compiler")
 import("lib.detect.find_tool")
 import("private.utils.executable_path")
 import("private.utils.toolchain", {alias = "toolchain_utils"})
@@ -38,18 +36,22 @@ function _get_buildir(package, opt)
     end
 end
 
--- map compiler flags
-function _map_compflags(package, langkind, name, values)
-    return compiler.map_flags(langkind, name, values, {target = package})
-end
-
--- map linker flags
-function _map_linkflags(package, targetkind, sourcekinds, name, values)
-    return linker.map_flags(targetkind, sourcekinds, name, values, {target = package})
-end
-
 -- get pkg-config, we need force to find it, because package install environments will be changed
 function _get_pkgconfig(package)
+    -- meson need fullpath pkgconfig
+    -- @see https://github.com/xmake-io/xmake/issues/5474
+    local dep = package:dep("pkgconf") or package:dep("pkg-config")
+    if dep then
+        local suffix = dep:is_plat("windows", "mingw") and ".exe" or ""
+        local pkgconf = path.join(dep:installdir("bin"), "pkgconf" .. suffix)
+        if os.isfile(pkgconf) then
+            return pkgconf
+        end
+        local pkgconfig = path.join(dep:installdir("bin"), "pkg-config" .. suffix)
+        if os.isfile(pkgconfig) then
+            return pkgconfig
+        end
+    end
     if package:is_plat("windows") then
         local pkgconf = find_tool("pkgconf", {force = true})
         if pkgconf then
@@ -153,7 +155,7 @@ function _insert_cross_configs(package, file, opt)
     elseif package:is_plat("windows") then
         local cpu
         local cpu_family
-        if package:is_arch("arm64") then
+        if package:is_arch("arm64", "arm64ec") then
             cpu = "aarch64"
             cpu_family = "aarch64"
         elseif package:is_arch("x86") then
@@ -210,10 +212,28 @@ function _get_configs_file(package, opt)
         if cc then
             file:print("c=['%s']", executable_path(cc))
         end
+
         local cxx = package:build_getenv("cxx")
         if cxx then
+            -- https://github.com/xmake-io/xmake/discussions/4979
+            if package:has_tool("cxx", "clang", "gcc") then
+                local dir = path.directory(cxx)
+                local name = path.filename(cxx)
+                name = name:gsub("clang$", "clang++")
+                name = name:gsub("clang%-", "clang++-") -- clang-xx
+                name = name:gsub("clang%.", "clang++.") -- clang.exe
+                name = name:gsub("gcc$", "g++")
+                name = name:gsub("gcc%-", "g++-")
+                name = name:gsub("gcc%.", "g++.")
+                if dir and dir ~= "." then
+                    cxx = path.join(dir, name)
+                else
+                    cxx = name
+                end
+            end
             file:print("cpp=['%s']", executable_path(cxx))
         end
+
         local ld = package:build_getenv("ld")
         if ld then
             file:print("ld=['%s']", executable_path(ld))
@@ -268,6 +288,14 @@ function _get_configs_file(package, opt)
         table.join2(cxxflags, _get_cflags_from_packagedeps(package, opt))
         table.join2(ldflags,  _get_ldflags_from_packagedeps(package, opt))
         table.join2(shflags,  _get_ldflags_from_packagedeps(package, opt))
+        -- add runtimes flags
+        for _, runtime in ipairs(package:runtimes()) do
+            if not runtime:startswith("M") then
+                table.join2(cxxflags, toolchain_utils.map_compflags_for_package(package, "cxx", "runtime", {runtime}))
+                table.join2(ldflags, toolchain_utils.map_linkflags_for_package(package, "binary", {"cxx"}, "runtime", {runtime}))
+                table.join2(shflags, toolchain_utils.map_linkflags_for_package(package, "shared", {"cxx"}, "runtime", {runtime}))
+            end
+        end
         if #cflags > 0 then
             file:print("c_args=['%s']", table.concat(cflags, "', '"))
         end
@@ -295,7 +323,7 @@ function _get_configs(package, configs, opt)
 
     -- add prefix
     configs = configs or {}
-    table.insert(configs, "--prefix=" .. package:installdir())
+    table.insert(configs, "--prefix=" .. (opt.prefix or package:installdir()))
     table.insert(configs, "--libdir=lib")
 
     -- set build type
@@ -316,10 +344,17 @@ function _get_configs(package, configs, opt)
         table.insert(configs, "-Db_sanitize=address")
     end
 
-    -- add vs_runtime flags
-    local vs_runtime = package:config("vs_runtime")
-    if package:is_plat("windows") and vs_runtime then
-        table.insert(configs, "-Db_vscrt=" .. vs_runtime:lower())
+    -- add vs runtimes flags
+    if package:is_plat("windows") then
+        if package:has_runtime("MT") then
+            table.insert(configs, "-Db_vscrt=mt")
+        elseif package:has_runtime("MTd") then
+            table.insert(configs, "-Db_vscrt=mtd")
+        elseif package:has_runtime("MD") then
+            table.insert(configs, "-Db_vscrt=md")
+        elseif package:has_runtime("MDd") then
+            table.insert(configs, "-Db_vscrt=mdd")
+        end
     end
 
     -- add cross file
@@ -340,7 +375,7 @@ end
 
 -- get msvc
 function _get_msvc(package)
-    local msvc = toolchain.load("msvc", {plat = package:plat(), arch = package:arch()})
+    local msvc = package:toolchain("msvc")
     assert(msvc:check(), "vs not found!") -- we need to check vs envs if it has been not checked yet
     return msvc
 end
@@ -353,41 +388,73 @@ end
 -- fix libname on windows
 function _fix_libname_on_windows(package)
     for _, lib in ipairs(os.files(path.join(package:installdir("lib"), "lib*.a"))) do
-        os.mv(lib, lib:gsub("(.+)\\lib(.-)%.a", "%1\\%2.lib"))
+        os.mv(lib, (lib:gsub("(.+)\\lib(.-)%.a", "%1\\%2.lib")))
     end
 end
 
 -- get cflags from package deps
 function _get_cflags_from_packagedeps(package, opt)
-    local result = {}
+    local values
     for _, depname in ipairs(opt.packagedeps) do
-        local dep = type(depname) ~= "string" and depname or package:dep(depname)
+        local dep = type(depname) ~= "string" and depname or package:librarydep(depname)
         if dep then
-            local fetchinfo = dep:fetch({external = false})
+            local fetchinfo = dep:fetch()
             if fetchinfo then
-                table.join2(result, _map_compflags(package, "cxx", "define", fetchinfo.defines))
-                table.join2(result, _map_compflags(package, "cxx", "includedir", fetchinfo.includedirs))
-                table.join2(result, _map_compflags(package, "cxx", "sysincludedir", fetchinfo.sysincludedirs))
+                if values then
+                    values = values .. fetchinfo
+                else
+                    values = fetchinfo
+                end
             end
         end
     end
+    -- @see https://github.com/xmake-io/xmake-repo/pull/4973#issuecomment-2295890196
+    local result = {}
+    if values then
+        if values.defines then
+            table.join2(result, toolchain_utils.map_compflags_for_package(package, "cxx", "define", values.defines))
+        end
+        if values.includedirs then
+            table.join2(result, toolchain_utils.map_compflags_for_package(package, "cxx", "includedir", values.includedirs))
+        end
+        if values.sysincludedirs then
+            table.join2(result, toolchain_utils.map_compflags_for_package(package, "cxx", "sysincludedir", values.sysincludedirs))
+        end
+    end
     return _translate_flags(package, result)
 end
 
 -- get ldflags from package deps
 function _get_ldflags_from_packagedeps(package, opt)
-    local result = {}
+    local values
     for _, depname in ipairs(opt.packagedeps) do
-        local dep = type(depname) ~= "string" and depname or package:dep(depname)
+        local dep = type(depname) ~= "string" and depname or package:librarydep(depname)
         if dep then
-            local fetchinfo = dep:fetch({external = false})
+            local fetchinfo = dep:fetch()
             if fetchinfo then
-                table.join2(result, _map_linkflags(package, "binary", {"cxx"}, "linkdir", fetchinfo.linkdirs))
-                table.join2(result, _map_linkflags(package, "binary", {"cxx"}, "link", fetchinfo.links))
-                table.join2(result, _map_linkflags(package, "binary", {"cxx"}, "syslink", fetchinfo.syslinks))
+                if values then
+                    values = values .. fetchinfo
+                else
+                    values = fetchinfo
+                end
             end
         end
     end
+    local result = {}
+    if values then
+        if values.linkdirs then
+            table.join2(result, toolchain_utils.map_linkflags_for_package(package, "binary", {"cxx"}, "linkdir", values.linkdirs))
+        end
+        if values.links then
+            table.join2(result, toolchain_utils.map_linkflags_for_package(package, "binary", {"cxx"}, "link", values.links))
+        end
+        if values.syslinks then
+            table.join2(result, toolchain_utils.map_linkflags_for_package(package, "binary", {"cxx"}, "syslink", values.syslinks))
+        end
+        if values.frameworks then
+            table.join2(result, toolchain_utils.map_linkflags_for_package(package, "binary", {"cxx"}, "framework", values.frameworks))
+        end
+    end
     return _translate_flags(package, result)
 end
 
@@ -499,20 +566,17 @@ end
 -- install package
 function install(package, configs, opt)
 
-    -- generate build files
+    -- do build
     opt = opt or {}
-    generate(package, configs, opt)
+    build(package, configs, opt)
 
     -- configure install
     local buildir = _get_buildir(package, opt)
     local argv = {"install", "-C", buildir}
-    if option.get("verbose") then
-        table.insert(argv, "-v")
-    end
 
-    -- do build and install
+    -- do install
     local meson = assert(find_tool("meson"), "meson not found!")
-    os.vrunv(meson.program, {"install", "-C", buildir}, {envs = opt.envs or buildenvs(package, opt)})
+    os.vrunv(meson.program, argv, {envs = opt.envs or buildenvs(package, opt)})
 
     -- fix static libname on windows
     if package:is_plat("windows") and not package:config("shared") then
